#!/usr/bin/env python
__version__ = '$Revision: 7470 $'.split()[1]
__date__ = '$Date: 2007-11-06 10:31:44 -0500 (Tue, 06 Nov 2007) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''

Connect to a socket and provide a service where multiple clients can
connect.  The program optionally logs the data stream to a file.
Migrated from ais-py in August 2007.

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2006
@since: 2006-May-04
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser

@todo: log rotation!!  Optionally compress the logs when done.  They compress well.
@todo: add udp in and udp out
@todo: line oriented mode so that
@todo: allow the feed in and the exports to be on different interfaces
@todo: rewrite this as a SocketServer?
@todo: optionally read data from a serial port
@todo: multicast out
@todo: timestamps in line mode
@todo: multiple socket inputs and multiple serial port inputs
@todo: use select for blocking IO
'''

#import sys, os, shutil
import time
import socket
import thread

######################################################################

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0
NEVER    = 0 

def addVerbosityOptions(parser):
    """
    Added the verbosity options to a parser
    """
    parser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                        help='how much information to give.  Specify multiple times to increase verbosity')
    parser.add_option('--verbosity',dest='verbosity',type='int',
                        help='Specify verbosity.  Should be in the range of '
                        +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')
    parser.add_option('--noisy',dest='verbosity', action='store_const', const=2*BOMBASTIC,
                        help='Go for the max verbosity ['+str(2*BOMBASTIC)+']')

######################################################################

def dateStr():
    '''
    String representing the day so that it sorts correctly
    @return: yyyy-mm-dd
    @rtype: str
    '''
    t=time.gmtime()
    d='%04d-%02d-%02d' % t[:3]
    return d


######################################################################
class PassThroughServer:
    '''Receive data from a socket and write the data to all clients that
    are connected.  Starts two threads and returns to the caller.
    '''
    def __init__(self,options):
	self.clients=[]
	self.options = options
	if options.logFile:
	    self.curLogFile=self.getLogFileName()
	    self.log=open(self.curLogFile,'a')
        else: self.log = None
	self.count=0

    def start(self):
	print 'starting threads'
	thread.start_new_thread(self.passdata,(self,))
	thread.start_new_thread(self.connection_handler,(self,))
	return

    def getLogFileName(self):
	'''Return the log file name.  Appends date if rotation is happenin

	FIX: update when the program can rotate at arbitrary times.
	'''
	if self.options.rotateLog:
	    return self.options.logFile+dateStr()
	return self.options.logFile

    def passdata(self,unused=None):
	'''Do not use this.  Call start() instead.

	@bug: how can I get rid of unused?
	'''
	print 'starting passthrough server'

	# remote is where our data comes from
	remote = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
	remote.connect((self.options.inHost,self.options.inPort))

	while 1:
	    time.sleep(.001) # Replace with select
	    # Go slower for debugging
	    if self.options.verbosity>BOMBASTIC: time.sleep(.5)
	    #else time.sleep(.01)  # Is this needed to reschedule?
	    # FIX: use select instead
	    
	    # Use this for code that executes every N reads
	    self.count += 1
	    if self.count%100==1:
		print '# TIME =',time.gmtime()
		print
		print '#  HOUR,MIN: ', time.gmtime()[3:5]
		print
		

	    if self.log and self.options.rotateLog:
		newLogFile=self.getLogFileName()
		if self.curLogFile!=newLogFile:
		    if self.options.verbosity>=TERSE:
			print 'ROTATING LOG: ',self.curLogFile, newLogFile
		    self.log.close()
		    self.curLogFile=newLogFile
		    self.log=open(self.curLogFile,'a')

	    m = remote.recv(10000)
	    if len(m)>0: 
		if self.log: 
		    self.log.write(m)  # Takes a few before it flushes
		if self.options.verbosity>TERSE: print m,
		for c in self.clients:
		    try:
			c.send(m)
		    except socket.error:
			print 'Client Disconnect'
			self.clients.remove(c)
	    elif self.options.verbosity>=VERBOSE: 
		print 'no data'
	
    def connection_handler(self,unused=None):
	'''Do not use this.  Call start() instead.  This listens for
	connections and adds the new socket to the clients list.

	@bug: how can I get rid of unused?
	'''
	print 'starting incoming connection receiver'

	serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	serversocket.bind((self.options.outHost, self.options.outPort))
	serversocket.listen(5)

	while 1:
	    (clientsocket, address) = serversocket.accept()
	    print 'connect from',clientsocket, address
	    self.clients.append(clientsocket)


######################################################################


if __name__=='__main__':
    from optparse import OptionParser

    # FIX: is importing __init__ safe?
    myparser = OptionParser(usage="%prog [options]",
                            version="%prog "+__version__ + " ("+__date__+")")

    myparser.add_option('-l','--log-file',dest='logFile',type='string',default=None,
			help='Write the stream through a log file [default: %default]')

    myparser.add_option('-i','--in-port',dest='inPort',type='int', default=31414,
			help='Where the data comes from [default: %default]')
    myparser.add_option('-I','--in-host',dest='inHost',type='string',default='localhost',
			help='What host to read data from [default: %default]')
#    myparser.add_option('--in-gethostname',dest='inHostname', action='store_true', default=False,
#			help='Where the data comes from [default: %default]')


    myparser.add_option('-o','--out-port', dest="outPort", type='int',default=31401,
			help='Where the data will be available to others [default: %default]')
    myparser.add_option('-O','--out-host',dest='outHost',type='string', default='localhost',
			help='What machine the source port is on [default: %default]')
    myparser.add_option('--out-gethostname',dest='outHostname', action='store_true', default=False,
			help='Use the default hostname ['+socket.gethostname()+']')

    # FIX: Change to allow the user to specify when logs will be rotated
    myparser.add_option('-r','--rotate', dest='rotateLog', default=False,
			action='store_true', help='turn on one a day log rotation.'+
			'Appends the date to the log')


    addVerbosityOptions(myparser)

    (options,args) = myparser.parse_args()

    if options.outHostname:
	options.outHost=socket.gethostname()

    if options.verbosity>=VERBOSE:
	print options


    pts = PassThroughServer(options)
    pts.start()
    
    v = options.verbosity
    del(options) # remove global to force self.options
    i = 0
    while 1:
	i+=1
	time.sleep(30)
	if v>=TRACE: print 'ping',i
